/* $Id: Gadgets.cc,v 1.11.2.5 1998/11/24 20:46:48 rousskov Exp $ */

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */


#include "AppDef.h"

#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/sysctl.h>
#include <netinet/in.h>  /* hton? on Linux */

#include "Gadgets.h"

static struct Checker {
	Checker() { Assert(sizeof(timeval) == 8); } // htonTime relies on that
} TheChecker;

char *StrDup(const char *str, int len) {

	if (!str) return 0;

	char *copy = new char [len+1];
	memcpy(copy, str, len);
	copy[len] = '\0';

	return copy;
}

const char *SubStr(const char *str, char del) {
	static char buf[1024];

	const char *p = strchr(str, del);
	unsigned sz = p - str;
	if (sz >= sizeof(buf)) sz = sizeof(buf)-1;
	memcpy(buf, str, sz);
	buf[sz] = '\0';
	return buf;
}

int xatoi(const char *s, int def = 0) { 
	if (!s)
		return def;
	char *ptr = 0;
	const int h = (int) strtol(s, &ptr, 10);
	return ptr == s ? def : h;
}

void configureStream(ostream &os, int prec) {
	os.precision(prec);
	os.setf(ios::fixed, ios::floatfield);
	os.setf(ios::showpoint, ios::basefield);
}

int GetMaxFD() {
	struct rlimit rl;
	if (getrlimit(RLIMIT_NOFILE, &rl) < 0)
		return -1;
	return (int)rl.rlim_max;
}

int SetMaxFD(int maxFD) {
	struct rlimit rl;
	rl.rlim_max = min(maxFD, GetMaxFD());
	rl.rlim_cur = rl.rlim_max;
	if (rl.rlim_cur >= 0)
		Assert(setrlimit(RLIMIT_NOFILE, &rl) >= 0);
	return rl.rlim_cur;
}

void hton16(char *buf) {
	short int i;
	memcpy(&i, buf, sizeof(i));
	i = (short int) htons(i);
	memcpy(buf, &i, sizeof(i));
}

void ntoh16(char *buf) {
	short int i;
	memcpy(&i, buf, sizeof(i));
	i = (short int) ntohs(i);
	memcpy(buf, &i, sizeof(i));
}

void hton32(char *buf) {
	int i;
	memcpy(&i, buf, sizeof(i));
	i = (int) htonl(i);
	memcpy(buf, &i, sizeof(i));
}

void ntoh32(char *buf) {
	int i;
	memcpy(&i, buf, sizeof(i));
	i = (int) ntohl(i);
	memcpy(buf, &i, sizeof(i));
}



// reads a line and checks if it matches the given string
bool GetConstLine(istream &is, const char *match) {
	char str[strlen(match)+1];
	is.getline(str, sizeof(str));
	return strcmp(str, match) == 0;
}
