// $Id: HttpMsg.cc,v 1.14.2.4 1998/11/17 20:54:53 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "AppDef.h"

#include <iostream.h>
#include <iomanip.h>

#include "HttpMsg.h"
#include "Log.h"
#include "Gadgets.h"

SafePtrDefn(HttpRepHdrPtr, HttpRepHeader)
SafePtrDefn(HttpReqHdrPtr, HttpReqHeader)

/* we often use this type */
typedef const char *cstr;

// returns a pointer to a character right next to the header or null
cstr EndOfHeaders(cstr s) {
	cstr eoh = strstr(s, "\n\r\n");
	if (eoh)
		return eoh + 3;
	eoh = strstr(s, "\n\n");
	if (eoh)
		return eoh + 2;
	return 0;
}


/* HttpMsgHeader */

bool HttpMsgHeader::parse(const char *buf) {
	cstr eofl = strchr(buf, '\n'); /* end of status-line or request-line */
	if (!eofl)
		return false;
	eofl++;

	cstr eoh = EndOfHeaders(eofl);
	if (!eoh)
		return false;
	theHdrSize = eoh - buf;

	// process some known headers
	cstr h;

	// detect xaction Id if any
	if ((h = strstr(eofl, "X-Xact: "))) {
		theXactId = atoi(h + 8);
		Assert(theXactId != 0);
	}

	/* ... more */
	return true;
}


/* HttpRepHeader */

HttpRepHeader::HttpRepHeader(): HttpMsgHeader(msgReply), theStatus(-1), theContLen(-1) {
}

bool HttpRepHeader::parse(const char *buf) {

	if (!HttpMsgHeader::parse(buf))
		return false;
	if (strncasecmp(buf, "HTTP/", 5) != 0)
		return false;
	// ...
	cstr status = strchr(buf, ' ');
	if (status)
		theStatus = atoi(status);
	else
		return false;
	if (cstr h = strstr(status, "Content-Length: ")) {
		theContLen = atoi(h + 16);
	} else
	if (cstr h = strstr(status, "Content-length: ")) {
		theContLen = atoi(h + 16);
	}
	// ...
	Notepad << "got status: " << theStatus <<  " clen: " << theContLen << ende;
	return true;
}


/* HttpReqHeader */

HttpReqHeader::HttpReqHeader(): HttpMsgHeader(msgRequest), 
	theUri(0), isIMS(false) {
}

HttpReqHeader::~HttpReqHeader() {
	delete[] theUri;
}

bool HttpReqHeader::parse(const char *buf) {
	Assert(!theUri);

	if (!HttpMsgHeader::parse(buf))
		return false;
	if (strncasecmp(buf, "GET", 3) != 0)
		return false;
	// ...
	cstr uri = strchr(buf+3, ' ');
	if (!uri)
		return false;
	uri++;

	// extract object id
	cstr idStr = strchr(uri, '_');
	theObjId = idStr ? atoi(idStr+1) : -1;

	cstr suffix = strchr(uri, ' ');
	if (!suffix)
		return false;
	theUri = StrDup(uri, suffix-uri);
	// ...
	Notepad << "got uri: " << theUri << endl;

	// IMS request?
	isIMS = strstr(suffix, "If-Modified-Since: ") != 0;
	return true;
}
