// $Id: Xaction.cc,v 1.18.2.6 1998/11/18 07:23:44 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include <string.h>

#include "Xaction.h"
#include "ConnInfo.h"
#include "PushPull.h"
#include "LocalStats.h"
#include "ErrHist.h"
#include "globals.h"
#include "Log.h"

/* XactCfg */

XactCfg::XactCfg():
	theContMaker(0), theContEater(0), theReader(0), theWriter(0) {
}

void XactCfg::destroy() {
	delete theContMaker;
	delete theContEater;
	delete theReader;
	delete theWriter;
}

XactCfg XactCfg::clone() const {
	XactCfg cfg;
	cfg.theContMaker = (ContThread*)theContMaker->clone();
	cfg.theContEater = (ContThread*)theContEater->clone();
	cfg.theReader = (XactIOThread*)theReader->clone();
	cfg.theWriter = (XactIOThread*)theWriter->clone();
	return cfg;
}

/* Xaction */

inline static
void threadPartnership(XactThread *th1, XactThread *th2) {
	th1->partner(th2);
	th2->partner(th1);
}

inline static
void contPartnership(ContThread *th1, ContThread *th2) {
	th1->contPartner(th2);
	th2->contPartner(th1);
}


Xaction::~Xaction() {
	if (theAddKidCount) // kludge: we managed to start PushPulls
		theParent->taskStopped(this);
	theCfg.destroy();
	if (theStart > 0 && ThePhaseStats->theRespTimeTrace && ThePhaseStats->theRespSizeTrace) { // XXX: big kludge
		ThePhaseStats->theRespTimeTrace->record((TheClock-theStart).usec());
		ThePhaseStats->theRespSizeTrace->record(theRespSize);
	}
}

Thread *Xaction::clone() const {
	Xaction *x = new Xaction();
	x->configure(theCfg.clone());
	return x;
}

void Xaction::configure(const XactCfg &aCfg) {
	theCfg = aCfg;
	// set partners
	threadPartnership(theCfg.theContMaker, theCfg.theWriter);
	threadPartnership(theCfg.theContEater, theCfg.theReader);
	contPartnership(theCfg.theContMaker, theCfg.theContEater);
}

// note: must be called before init() and after configure()
void Xaction::conn(const ConnMx &anInMx, const ConnMx &anOutMx) {
	theInMx = anInMx;
	theOutMx = anOutMx;
	theStart = TheClock;
	if (TheOpt.theConnEstTimout > 0)
		sleepFor(TheOpt.theConnEstTimout);
}

void Xaction::run() {
	// (moved to init(): theParent->taskStarted(this);
	startPushPull(theCfg.theReader, theCfg.theContEater, theInMx);
	startPushPull(theCfg.theContMaker, theCfg.theWriter, theOutMx);
	/* we no longer control these */
	theCfg.theReader = theCfg.theWriter = 0;
	theCfg.theContMaker = theCfg.theContEater = 0;
}

void Xaction::startPushPull(XactThread *pusher, XactThread *puller, ConnMx &conn) {
	PushPull *pp = new PushPull(pusher, puller);
	pp->conn(conn);
	execKid(pp);
}

void Xaction::wakeUp(const Alarm &) {
	// currently the only reason for an alarm is a connection timeout
	// XXX: may not work with persistent connections
	if (theInMx && !theInMx->connected()) {
		SysLog << "connection timeout in " << TheClock - theStart << ende;
		beActive(thDone);
		theInMx->close();
		theInMx.releaseVoid();
		TheErrHist[E_CONN_EST_TOUT]++;
	}
}
