// $Id: ContThread.h,v 1.10.2.1 1998/09/22 07:44:04 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef _CONT_THREAD_H_
#define _CONT_THREAD_H_

#include "XactThread.h"
#include "HttpMsg.h"

// a storage for content to pass between threads
class ContRec {
	public:
		void req(const HttpReqHdrPtr &aReqHdr);
		void rep(const HttpRepHdrPtr &aRepHdr);

	public:
		HttpReqHdrPtr theReqHdr;
		HttpRepHdrPtr theRepHdr;
};

inline
ostream &operator <<(ostream &os, const ContRec &cr) { return os << cr.theReqHdr << ',' << cr.theRepHdr; }

SafePtrDecl(ContThreadPtr, ContThread);

// common class for content eaters and makers
class ContThread: public XactThread {
	public:
		enum ParseState { psWaitHead, psWaitCBlock, psWaitBody, psDone };
		enum ContState { csNone, csValid, csVoid };

	public:
		ContThread(): theOtherContState(csNone) {}

		void contPartner(ContThread *aContPartner) { theContPartner.assign(aContPartner->useRec(), aContPartner); }

		// ``rpc'' handler
		virtual void otherContChanged(const ContRec &) = 0;

		virtual void run();

	protected:
		virtual bool enough() const { return theOtherContState == csVoid || XactThread::enough(); }

		virtual void doContent() = 0;

		void sendContChange();

	protected:
		ContThreadPtr theContPartner;
		ContState theOtherContState;

		ContRec theCont;
};

#endif
