#ifndef _HTTP_MSG_H_
#define _HTTP_MSG_H_ "$Id: HttpMsg.h,v 1.11.2.2 1998/11/05 18:41:20 rousskov Exp $"

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "AppDef.h"

#include "Size.h"
#include "SafePtr.h"

enum HttpMsgType { msgNone, msgReply, msgRequest };

// http msg header
class HttpMsgHeader {
	public:
		HttpMsgHeader(HttpMsgType aType): theType(aType), theObjId(-1), theXactId(-1) {}

		HttpMsgType type() const { return theType; }
		int objId() const { return theObjId; }
		int xactId() const { return theXactId; }
		Size hdrSize() const { return theHdrSize; }

		void objId(int anId) { theObjId = anId; }
		void xactId(int anId) { theXactId = anId; }
		void hdrSize(Size aSize) { theHdrSize = aSize; }

		bool parse(const char *buf);

	protected:
		Size theHdrSize;   // with request-line or status-line
		HttpMsgType theType;
		int theObjId;      // object (resource) id passed in the uri
		int theXactId;     // xaction id passed in X-Xact: header
};

// http reply header
class HttpRepHeader: public HttpMsgHeader {
	public:
		HttpRepHeader();

		bool noBody() const { return status() == 304; }
		int status() const { return theStatus; }
		Size contLen() const { return theContLen; }

		bool parse(const char *buf);

	protected:
		int theStatus;
		Size theContLen;
};

SafePtrDecl(HttpRepHdrPtr, HttpRepHeader);

// http reply header
class HttpReqHeader: public HttpMsgHeader {
	public:
		HttpReqHeader();
		~HttpReqHeader();

		const char *uri() const { return theUri; }
		bool ims() const { return isIMS; }

		bool parse(const char *buf);

	protected:
		char *theUri;
		bool isIMS;
};

SafePtrDecl(HttpReqHdrPtr, HttpReqHeader);


// returns a pointer to a character right next to the header or null
extern const char *EndOfHeaders(const char *buf);

#endif
