#ifndef _STATS_H_
#define _STATS_H_ "$Id: Stats.h,v 1.4.2.3 1998/11/08 04:56:47 rousskov Exp $"

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "AppDef.h"

#include <math.h>

// simple counter
class Count {
	public:
		Count(): theCount(0) {}

		int operator ++() { return ++theCount; }
		int operator ++(int) { return theCount++; }

		operator int() const { return theCount; }

	protected:
		int theCount;
};

// collects various single point statistics (min, max, avg, etc.)
// most methods return (-1) if statistics is not available
class SampleStat {
	public:
		SampleStat() { reset(); }

		void reset();

		int count() const { return theCount; }
		double sum() const { return theSum; }
		double min() const { return theMin; }
		double max() const { return theMax; }
		double avg() const { return theCount ? theSum/theCount : -1; }
		double var() const;
		double stdDev() const { return (theCount > 1) ? sqrt(var()) : -1; }
		double relDev() const { return (theCount > 1 && theSum != 0.0) ? stdDev()/avg() : -1; }

		void record(double v);

		void report(ostream &os) const;

	protected:
		int theCount;
		double theSum;
		double theSqSum;
		double theMax;
		double theMin;
};

#endif
